/***************************************************************************
                          headingsdialog.cpp  -  description
                             -------------------
    begin                : Sam Nov 29 2003
    copyright            : (C) 2003 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "headingsdialog.h"
#include <QCheckBox>
#include <QComboBox>
#include <QMessageBox>
#include <QString>
#include <QStackedWidget>
#include <QLine>

HeadingsDialog::HeadingsDialog(QWidget *parent) : QDialog(parent)
{
  setupUi(this);
  twoside = false;

  connect(checkBoxHsl, SIGNAL(toggled(bool)), lineHsl, SLOT(setShown(bool)));
  connect(checkBoxFsl, SIGNAL(toggled(bool)), lineFsl, SLOT(setShown(bool)));
  connect(checkBoxHslD, SIGNAL(toggled(bool)), lineHslD, SLOT(setShown(bool)));
  connect(checkBoxFslD, SIGNAL(toggled(bool)), lineFslD, SLOT(setShown(bool)));

  connect(buttonClear, SIGNAL(clicked()), this, SLOT(slotClear()));
  connect(buttonRestore, SIGNAL(clicked()), this, SLOT(slotReset()));
}

HeadingsDialog::~HeadingsDialog(){
}


void HeadingsDialog::slotReset()
{
  if (QMessageBox::question(this, tr("Restore"),
                                  tr("Do you really want to restore the initial setup?"),
                                  tr("&Yes"), tr("&No") )) return;
  const QStringList lst = listSettings;
  setOptions(lst);
}

void HeadingsDialog::slotClear()
{
  if (QMessageBox::question(this, tr("Clear"),
                                  tr("Do you really want to clear all entries?"),
                                  tr("&Yes"), tr("&No") )) return;
  comboBoxLeh->setCurrentIndex(0);
  comboBoxCeh->setCurrentIndex(0);
  comboBoxReh->setCurrentIndex(0);
  comboBoxLoh->setCurrentIndex(0);
  comboBoxCoh->setCurrentIndex(0);
  comboBoxRoh->setCurrentIndex(0);
  comboBoxLef->setCurrentIndex(0);
  comboBoxCef->setCurrentIndex(0);
  comboBoxRef->setCurrentIndex(0);
  comboBoxLof->setCurrentIndex(0);
  comboBoxCof->setCurrentIndex(0);
  comboBoxRof->setCurrentIndex(0);

  comboBoxIh->setCurrentIndex(0);
  comboBoxCh->setCurrentIndex(0);
  comboBoxOh->setCurrentIndex(0);
  comboBoxIf->setCurrentIndex(0);
  comboBoxCf->setCurrentIndex(0);
  comboBoxOf->setCurrentIndex(0);

  checkBoxHsl->setChecked(false);
  checkBoxFsl->setChecked(false);
  checkBoxHslD->setChecked(false);
  checkBoxFslD->setChecked(false);
}

void HeadingsDialog::setTwoside(bool b) 
{
  twoside = b;
  if (twoside) {
    stackedWidget->setCurrentIndex(1);
    setWindowTitle("Scrpage2 Headings (twoside)");
  }
  else {
    stackedWidget->setCurrentIndex(0);
    setWindowTitle("Scrpage2 Headings (oneside)");
  }
}


// parse a string list containing all current items for combo
// boxes check status of check boxes.
// format of the string list:
// Pos 0: status of headsepline (0 = false, 1 = true)
// Pos 1: status of footsepline (0 = false, 1 = true)
// Pos 2--19: current items of combo boxes
void HeadingsDialog::setOptions(const QStringList &lst)
{
  listSettings = lst;
  int d = 0;
  for (int i = 0; i < 20; i++) {
    if (i >= lst.count()) break;
    d = lst[i].toInt();
    if (d > 3) d = 0;

    if (i == 0) {
      if (d == 0) {checkBoxHsl->setChecked(false);     checkBoxHslD->setChecked(false);}
      if (d == 1) {checkBoxHsl->setChecked(true);      checkBoxHslD->setChecked(true);}
    }
    if (i == 1) {
      if (d == 0) {checkBoxFsl->setChecked(false);     checkBoxFslD->setChecked(false);}
      if (d == 1) {checkBoxFsl->setChecked(true);      checkBoxFslD->setChecked(true);}
    }
    if (i == 2)  comboBoxLeh->setCurrentIndex(d);
    if (i == 3)  comboBoxCeh->setCurrentIndex(d);
    if (i == 4)  comboBoxReh->setCurrentIndex(d);
    if (i == 5)  comboBoxLoh->setCurrentIndex(d);
    if (i == 6)  comboBoxCoh->setCurrentIndex(d);
    if (i == 7)  comboBoxRoh->setCurrentIndex(d);
    if (i == 8)  comboBoxLef->setCurrentIndex(d);
    if (i == 9)  comboBoxCef->setCurrentIndex(d);
    if (i == 10) comboBoxRef->setCurrentIndex(d);
    if (i == 11) comboBoxLof->setCurrentIndex(d);
    if (i == 12) comboBoxCof->setCurrentIndex(d);
    if (i == 13) comboBoxRof->setCurrentIndex(d);

    if (i == 14) comboBoxIh->setCurrentIndex(d);
    if (i == 15) comboBoxCh->setCurrentIndex(d);
    if (i == 16) comboBoxOh->setCurrentIndex(d);
    if (i == 17) comboBoxIf->setCurrentIndex(d);
    if (i == 18) comboBoxCf->setCurrentIndex(d);
    if (i == 19) comboBoxOf->setCurrentIndex(d);
  }
}

// format of the string list:
// Pos 0: status of headsepline (0 = false, 1 = true)
// Pos 1: status of footsepline (0 = false, 1 = true)
// Pos 2--19: current items of combo boxes
QStringList HeadingsDialog::getOptions()
{
  QStringList lst;
  if (twoside) {
    if (checkBoxHslD->isChecked()) lst.push_back("1");
    else lst.push_back("0");
    if (checkBoxFslD->isChecked()) lst.push_back("1");
    else lst.push_back("0");
  }
  else {
    if (checkBoxHsl->isChecked()) lst.push_back("1");
    else lst.push_back("0");
    if (checkBoxFsl->isChecked()) lst.push_back("1");
    else lst.push_back("0");
  }

  QString s;
  lst.push_back(s.setNum(comboBoxLeh->currentIndex()));
  lst.push_back(s.setNum(comboBoxCeh->currentIndex()));
  lst.push_back(s.setNum(comboBoxReh->currentIndex()));
  lst.push_back(s.setNum(comboBoxLoh->currentIndex()));
  lst.push_back(s.setNum(comboBoxCoh->currentIndex()));
  lst.push_back(s.setNum(comboBoxRoh->currentIndex()));
  lst.push_back(s.setNum(comboBoxLef->currentIndex()));
  lst.push_back(s.setNum(comboBoxCef->currentIndex()));
  lst.push_back(s.setNum(comboBoxRef->currentIndex()));
  lst.push_back(s.setNum(comboBoxLof->currentIndex()));
  lst.push_back(s.setNum(comboBoxCof->currentIndex()));
  lst.push_back(s.setNum(comboBoxRof->currentIndex()));

  lst.push_back(s.setNum(comboBoxIh->currentIndex()));
  lst.push_back(s.setNum(comboBoxCh->currentIndex()));
  lst.push_back(s.setNum(comboBoxOh->currentIndex()));
  lst.push_back(s.setNum(comboBoxIf->currentIndex()));
  lst.push_back(s.setNum(comboBoxCf->currentIndex()));
  lst.push_back(s.setNum(comboBoxOf->currentIndex()));
  return lst;
}
