/***************************************************************************
                          marginprev.cpp  -  description
                             -------------------
    begin                : Mon Feb 23 2004
    copyright            : (C) 2004 by Nicola D�elin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "marginprev.h"
#include <QPainter>
#include <QPixmap>
#include <QColor>
#include <QMatrix>
#include <QPalette>

MarginPrev::MarginPrev(QWidget *parent, Qt::WindowFlags f ) : QFrame(parent,f) 
{
  #ifdef Q_OS_MACX
    color = QColor(200,200,200);
  #else
    color = palette().window().color();
  #endif

  setAttribute(Qt::WA_OpaquePaintEvent);

  wi = 210.0;
  he = 297.0;
  di = 5.0;
  bc = 0.0;
  space = 10.0;
  pages = 2.0;
  divisor = 14.0;
}

MarginPrev::~MarginPrev(){
}

void MarginPrev::setMargin(const float &w, const float &h, 
                           const float &div, const float &bcor)
{
  // these variables are globally defined
  wi = w;
  he = h;
  di = div;
  bc = bcor;

  space = 10.0;
  pages = 2.0;
  pmargin = 2;

  if (!twoside) {
    space = 0.0;
    pages = 1.0;
  }

  // these conditions will lead to divisions by 0:
  if (width() == (int)space) return;
  if (width() == 0) return;
  if (height() == 0) return;

  // make sure the paper preview fits in the widget (landscape / portrait)
  if ((he / float(height() - 2*pmargin)) >= ((pages * wi) / float(width() - space))) {
    divisor = he / float(height() - 2*pmargin);
  } else { 
    divisor = (pages * wi) / float(width() - space);
  }

  // will lead to divisions by 0:
  if (divisor == 0.0) return;

  // define the rectangle representing the page
  paper.setX((width() - 2*pmargin - int(wi / divisor))/2);
  paper.setY((height() - 2*pmargin - int(he / divisor))/2);
  paper.setWidth(int(wi / divisor));
  paper.setHeight(int(he / divisor));

  // define the rectangle representing the write area
  if (twoside) writeArea.setX(paper.x() + int((bc + 1.0 * float((wi-bc) / di)) / divisor));
  else writeArea.setX(paper.x() + int((bc + 1.5 * float((wi-bc) / di)) / divisor));

  writeArea.setY(paper.y() + int((he / di) / divisor));
  writeArea.setWidth(int((wi - bc - 3.0 * float((wi - bc) / di)) / divisor) + 1);
  writeArea.setHeight(int((he - 3*(he / di)) / divisor) + 1);

  // define the rectangle representing the BCOR
  if (bc > 0) {
    bcorRect.setX(paper.x()+1);
    bcorRect.setY(paper.top()+1);
    bcorRect.setWidth(int(bc / divisor));
    bcorRect.setHeight(paper.height() - 2);
  }

  update();
}

void MarginPrev::clear()
{
}

void MarginPrev::paintEvent(QPaintEvent *)
{
  QBrush fill(Qt::gray);
  fill.setStyle(Qt::DiagCrossPattern);
  QMatrix m(-1.0, 0.0, 0.0, 1.0, 0.0, 0.0);;

  // draw widget, but no calculation (all done before)
  QPainter p(this);
    p.fillRect(0, 0, width(), height(), color);

    if (twoside) {
      // draw the right page
      p.translate(pmargin + ((float)paper.width() + space)/2.0, pmargin);
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left(), paper.top()+1, writeArea.left(), paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right(), writeArea.top());
      p.drawLine(writeArea.right(), paper.top()+1, writeArea.right(), paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right(), writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right(), paper.top()+1, bcorRect.right(), paper.bottom());
        p.setBrush(fill);
        p.setPen(Qt::NoPen);
        p.drawRect(bcorRect);
      }

      // invert and draw the left page
      p.setWorldMatrix(m);
      p.translate((((float)paper.width() + space)/2.0) - width() - pmargin, pmargin );
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left()+1, paper.top()+1, writeArea.left()+1, paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right(), writeArea.top());
      p.drawLine(writeArea.right()+1, paper.top()+1, writeArea.right()+1, paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right(), writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right()+1, paper.top()+1, bcorRect.right()+1, paper.bottom());
        p.setBrush(fill);
        p.setPen(Qt::NoPen);
        p.drawRect(bcorRect);
      }
    }
    else {
      p.translate(pmargin, pmargin);
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left(), paper.top()+1, writeArea.left(), paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right(), writeArea.top());
      p.drawLine(writeArea.right(), paper.top()+1, writeArea.right(), paper.bottom());
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right(), writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right(), paper.top()+1, bcorRect.right(), paper.bottom());
        p.setBrush(fill);
        p.setPen(Qt::NoPen);
        p.drawRect(bcorRect);
      }
    }
}

void MarginPrev::resizeEvent(QResizeEvent *)
{
  setMargin(wi, he, di, bc);
}

void MarginPrev::setTwoside(bool b)
{
  twoside = b;
  setMargin(wi, he, di, bc);
}

