/***************************************************************************
                          markupdialog.cpp  -  description
                             -------------------
    begin                : Mon Dez 29 2003
    copyright            : (C) 2003 by Nicola Doebelin
    email                : adoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "markupdialog.h"
#include <QInputDialog>
#include <QWhatsThis>
#include <QListWidget>
#include <QListWidgetItem>

MarkupDialog::MarkupDialog(QWidget *parent) : QDialog(parent)
{
	setupUi(this);

	connect(buttonHelp, SIGNAL(clicked()), this, SLOT(whatsThis()));
	connect(buttonAdd, SIGNAL(clicked()), this, SLOT(addItem()));
	connect(buttonDelete, SIGNAL(clicked()), this, SLOT(removeItem()));
	connect(buttonStd, SIGNAL(clicked()), this, SLOT(defaultEntries()));
	connect(listWidgetMain, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(modifyItem(QListWidgetItem*)));
}

MarkupDialog::~MarkupDialog()
{
}


void MarkupDialog::addItem()
{
  bool ok;
  QString s = QInputDialog::getText(this, tr("New Macro"), tr("Enter new Command:"), 
                                    QLineEdit::Normal, "\\newcommand*{}{}", &ok);
  if (ok && !s.isEmpty()) listWidgetMain->addItem(s);
}

void MarkupDialog::removeItem()
{
  if (!listWidgetMain->count()) return;

  int i = listWidgetMain->currentRow();
  QListWidgetItem *it = listWidgetMain->takeItem(i);
  delete it;
}

void MarkupDialog::modifyItem(QListWidgetItem *it)
{
  bool ok;
  QString t = QInputDialog::getText(this, tr("Modify Macro"), tr("Enter Command:"), 
                                    QLineEdit::Normal, it->text(), &ok);

  if (ok && !t.isEmpty()) it->setText(t);
}

void MarkupDialog::defaultEntries()
{
  QStringList lst;
  lst.push_back("\\newcommand*{\\FigB}{fig.}      % \"fig.\" as used in brackets");
  lst.push_back("\\newcommand*{\\FigT}{figure}    % \"figure\" as used in running text");
  lst.push_back("\\newcommand*{\\FigsT}{figures}  % \"figures\" (plural) in running text");
  lst.push_back("\\newcommand*{\\TabB}{tab.}      % \"tab.\" as used in brackets");
  lst.push_back("\\newcommand*{\\TabT}{table}     % \"table\" as used in running text");
  lst.push_back("\\newcommand*{\\TabsT}{tables}   % \"tables\" (plural) used in running text");

  listWidgetMain->addItems(lst);
}


/* public slots */

QStringList MarkupDialog::getList()
{
  QStringList lst;
  for (int i = 0; i < listWidgetMain->count(); i++) {
    lst.push_back(listWidgetMain->item(i)->text());
  }
  return lst;
}

void MarkupDialog::setList(const QStringList &l) 
{
  listWidgetMain->addItems(l);
}


void MarkupDialog::whatsThis()
{
  QWhatsThis::enterWhatsThisMode();
}
