/***************************************************************************
                          pkgdialog.cpp  -  description
                             -------------------
    begin                : Don Jan 8 2004
    copyright            : (C) 2007 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pkgdialog.h"
#include <QApplication>
#include <QDebug>
#include <QDir>
#include <QFileInfo>
#include <QFont>
#include <QFontMetrics>
#include <QLabel>
#include <QList>
#include <QPushButton>
#include <QDialogButtonBox>
#include <QStringList>
#include <QListWidget>
#include <QListWidgetItem>
#include <QMessageBox>

PkgDialog::PkgDialog(QWidget *parent) : QDialog(parent)
{
  setupUi(this);

  abort = false;
  scanning = false;
  file_suffix = "sty";

  font = textLabelDir->font();

  connect(this, SIGNAL(styleFound(QString)), this, SLOT(insertStyle(QString)));
  connect(buttonScan, SIGNAL(clicked()), this, SLOT(slotScan()));
  connect(buttonAdd, SIGNAL(clicked()), this, SLOT(slotAddManually()));
  connect(buttonRemove, SIGNAL(clicked()), this, SLOT(slotRemoveManually()));
}

PkgDialog::~PkgDialog(){
}

void PkgDialog::setDirs(const QString &t, const QString &l)
{
  tdir = t;
  ltdir = l;
}

void PkgDialog::slotScan()
{
  if (scanning) {
    abortScanning();
    return;
  }

  QDir dt(tdir);
  QDir dl(ltdir);

  bool dirs_ok = true;
  if (!dt.exists() && !dl.exists()) dirs_ok = false;
  if (tdir.isEmpty() && ltdir.isEmpty()) dirs_ok = false;

  if (!dirs_ok) {
    QMessageBox::information(this, tr("No Texmf directories found"), tr("Please enter valid texmf and \nlocaltexmf directories in the preferences."));
    return;
  }

  abort = false;
  scanning = true;
  toggleButtonState(false);

  listWidget->clear();

  searchFiles(tdir);
  searchFiles(ltdir);

  listWidget->sortItems(Qt::AscendingOrder);
  textLabelDir->setText("");
  toggleButtonState(true);
}

void PkgDialog::searchFiles(QString dir)
{
  if (abort) return;

  QDir d(dir);

  QStringList t, s;
  QFileInfo f;
  QFontMetrics fm(font);

  textLabelDir->setText(fm.elidedText(dir, Qt::ElideMiddle, textLabelDir->width() - 10));
  qApp->processEvents();
  d.setFilter(QDir::Dirs | QDir::Files | QDir::NoSymLinks);
  d.setSorting(QDir::Name);

  t = d.entryList();

  for (QStringList::Iterator it = t.begin(); it != t.end(); ++it) {
    if ((*it == "..") || (*it == ".")) continue;
    if (abort) break;

    f.setFile(d.absolutePath() + "/" + *it);

    if (f.isFile()) {
      if (f.suffix() == file_suffix) {
        emit styleFound(f.completeBaseName());
      }
    }

    if (f.isDir()) {
      searchFiles(f.absoluteFilePath());
    }
  }
}

void PkgDialog::insertStyle(QString s)
{
  listWidget->addItem(s);
}

int PkgDialog::listWidgetCount()
{
  return listWidget->count();
}

QListWidgetItem * PkgDialog::takeItem(int i)
{
  return listWidget->takeItem(i);
}

QString PkgDialog::lineEditText()
{
  return lineEdit->text();
}

void PkgDialog::slotAddManually()
{
  if (lineEdit->text().isEmpty()) return;
  listWidget->addItem(lineEdit->text());
}

void PkgDialog::slotRemoveManually()
{
  QList<QListWidgetItem *> lst = listWidget->selectedItems();

  while (!lst.isEmpty()) {
    delete lst.takeFirst();
  }
}

void PkgDialog::abortScanning()
{
  abort = true;
  scanning = false;
  toggleButtonState(true);
}

void PkgDialog::toggleButtonState(bool b)
{
  buttonAdd->setEnabled(b);
  buttonRemove->setEnabled(b);
  buttonBox->setEnabled(b);
}

void PkgDialog::setSuffix(const QString &s)
{
  file_suffix = s;
}

void PkgDialog::setTitle(const QString &s)
{
  setWindowTitle(s);
}

void PkgDialog::setLabel(const QString &s)
{
  textLabelName->setText(s);
}

void PkgDialog::setItems(const QStringList &l)
{
  listWidget->clear();
  listWidget->addItems(l);
}

QStringList PkgDialog::getItems()
{
  QStringList lst;
  for (int i = 0; i < listWidget->count(); ++i) {
    lst.push_back(listWidget->item(i)->text());
  }

  return lst;
}

