/***************************************************************************
                          template.h  -  description
                             -------------------
    begin                : Mon Jul 24 16:47:18 CET 2005
    copyright            : (C) 2005 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEMPLATE_H
#define TEMPLATE_H

#include <QDateTime>
#include <QString>
#include <QStringList>

class Template{
  public:
    Template(QString n = QString::null);
    ~Template();

    /* reads options from a stringlist, each field containing
       a value */
    void setFromStringList(const QStringList &);

    /* reads options from a stringlist, each odd entry contains
       a value, even entries contain the description of the following
       value */
    void setFromLongStringList(const QStringList &);

    /* write template to string list, even fields contain descriptions */
    QStringList writeToStringList();

    /**
     *   Set functions:
     **/

    void setName(const QString &n) {
      _name = n;
    };

    void setTitle(const QString &t, const QString &a, const QDate &d, bool sdt) {
      _title = t; 
      _author = a;
      _date = d;
      _sdate = sdt;
     };

    void setDocClass(int dc, int ps, int fs, int ff, int of, int en, int la, bool ko) {
      _docClass = dc;
      _paperSize = ps;
      _fontSize = fs;
      _fontFam = ff;
      _outputFormat = of;
      _encoding = en;
      _language = la;
      _koma = ko;
    };

    void setCustomMarkup(bool b, const QStringList &l) {
      _setCustomMarkup = b;
      _customMarkup = l;
    };

    void setTitlePage(int bt, QString a = QString::null, QString b = QString::null,
                      QString c = QString::null, QString d = QString::null,
                      QString e = QString::null, QString f = QString::null,
                      QString g = QString::null, QString h = QString::null) {
      _titlePButton = bt;
      _head = a;
      _subject = b;
      _publisher = c;
      _thanks = d;
      _extra = e;
      _dedication = f;
      _top = g;
      _bottom = h;
    };

    void setPageLayout(int d, int c, int o, int l, int p, int fpf, bool s) {
      _duplex = d;
      _columns = c;
      _orientation = o;
      _lineSpacing = l;
      _pageStyle = p;
      _floatPageFrac = fpf;
      _sloppy = s;
    }

    void setLists(bool a, bool b, bool c, bool d, bool e) {
      _toc = a;
      _lof = b;
      _lot = c;
      _index = d;
      _bibliography = e;
    }

    void setListsToc(bool lt, bool it, bool bt, int dep) {
      _lofToc = lt;
      _idxToc = it;
      _bibToc = bt;
      _tocdepth = dep;
    }

    void setKoma(int d, int b, int hs, int cap) {
      _div = d;
      _bcor = b;
      _headingSize = hs;
      _captions = cap;
    }

    void setBibtex(int type, int style, const QString &db, const QStringList &stList) {
      _bibType = type;
      _bibStyle = style;
      _bibDb = db;
      _bibStyleList = stList;
    }

    void setPackages(const QStringList &avail, const QStringList &loaded) {
      _pkgsAvailable = avail;
      _pkgsLoaded = loaded;
    }

    /**
     *   Get functions:
     **/

    QString name() {return _name;};
    QString title() {return _title;};
    QString author() {return _author;};

    QStringList customMarkup() {return _customMarkup;};

    int documentClass() {return _docClass;};
    int paperSize() {return _paperSize;};
    int fontSize() {return _fontSize;};
    int fontFam() {return _fontFam;};
    int outputFormat() {return _outputFormat;};
    int encoding() {return _encoding;};
    int language() {return _language;};

    bool koma() {return _koma;};
    bool useCustomMarkup() {return _setCustomMarkup;};

    QDate date() {return _date;};
    bool staticDate() {return _sdate;};

    int titlePageButton() {return _titlePButton;};

    QString titleHead() {return _head;};
    QString subject() {return _subject;};
    QString publisher() {return _publisher;};
    QString thanks() {return _thanks;};
    QString extra() {return _extra;};
    QString dedication() {return _dedication;};
    QString backsideTop() {return _top;};
    QString backsideBottom() {return _bottom;};

    int duplex() {return _duplex;};
    int columns() {return _columns;};
    int orientation() {return _orientation;};
    int lineSpacing() {return _lineSpacing;};
    int pageStyle() {return _pageStyle;};
    int floatPageFraction() {return _floatPageFrac;};

    bool sloppy() {return _sloppy;};

    bool toc() {return _toc;};
    bool lof() {return _lof;};
    bool lot() {return _lot;};
    bool index() {return _index;};
    bool bibliography() {return _bibliography;};

    bool lofToToc() {return _lofToc;};
    bool idxToToc() {return _idxToc;};
    bool bibToToc() {return _bibToc;};

    int tocDepth() {return _tocdepth;};

    int div() {return _div;};
    int bcor() {return _bcor;};
    int headingSize() {return _headingSize;};
    int captions() {return _captions;};

    int bibtexType() {return _bibType;};
    int bibtexStyle() {return _bibStyle;};

    QString bibtexDatabase() {return _bibDb;};

    QStringList bibtexStylesList() {return _bibStyleList;};

    QStringList packagesAvailable() {return _pkgsAvailable;};
    QStringList packagesLoaded() {return _pkgsLoaded;};


  private:
    /**
     *   private functions
     **/

    QString bts(bool);
    bool stb(const QString &);

    /**
     *   private variables
     **/

    QString inlineSep;

    QString _name,
            _title,
            _author;

    QStringList _customMarkup;

    int _docClass,
        _paperSize,
        _fontSize,
        _fontFam,
        _outputFormat,
        _encoding,
        _language;

    bool _koma,
         _setCustomMarkup;

    QDate _date;

    bool _sdate;

    int _titlePButton;

    QString _head,
            _subject,
            _publisher,
            _thanks,
            _extra,
            _dedication,
            _top,
            _bottom;

    int _duplex,
        _columns,
        _orientation,
        _lineSpacing,
        _pageStyle,
        _floatPageFrac;

    bool _sloppy;

    bool _toc,
         _lof,
         _lot,
         _index,
         _bibliography;

    bool _lofToc,
         _idxToc,
         _bibToc;

    int _tocdepth;

    int _div,
        _bcor,
        _headingSize,
        _captions;

    int _bibType,
        _bibStyle;

    QString _bibDb;

    QStringList _bibStyleList;

    QStringList _pkgsAvailable,
                _pkgsLoaded;
};

#endif
