/***************************************************************************
                          templatedialog.cpp  -  description
                             -------------------
    begin                : Sam Nov 29 16:47:18 CET 2003
    copyright            : (C) 2003 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "templatedialog.h"

#include "template.h"

#include <QInputDialog>
#include <QLineEdit>
#include <QListWidget>
#include <QListWidgetItem>
#include <QString>
#include <QStringList>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QVector>

TemplateDialog::TemplateDialog(QWidget *parent) : QDialog(parent)
{
	setupUi(this);

	templateList = new QVector<Template>();

	connect(listWidgetTemplates, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(currentItemChanged(QListWidgetItem *)));
	connect(listWidgetTemplates, SIGNAL(itemActivated(QListWidgetItem *)), this, SLOT(rename(QListWidgetItem *)));
	connect(buttonRename, SIGNAL(clicked()), this, SLOT(rename()));
	connect(buttonRemove, SIGNAL(clicked()), this, SLOT(remove()));
}

TemplateDialog::~TemplateDialog()
{
}

void TemplateDialog::rename()
{
  rename(listWidgetTemplates->currentItem());
}

void TemplateDialog::rename(QListWidgetItem *itm)
{
  int idx = listWidgetTemplates->row(itm);
  if (idx >= templateList->size()) return;
  QString txt = itm->text();
  bool ok;

  QString nm = QInputDialog::getText(this,
                                     tr("Rename Template"),
                                     tr("Enter new Name:"),
                                     QLineEdit::Normal,
                                     txt, &ok);

  if (!ok || nm.isEmpty()) return;

  itm->setText(nm);
  Template t = templateList->at(idx);
  t.setName(nm);
  templateList->replace(idx, t);
}

void TemplateDialog::remove()
{
  QListWidgetItem *itm = listWidgetTemplates->currentItem();
  int idx = listWidgetTemplates->row(itm);

  if (idx >= templateList->count()) return;

  delete itm;
  templateList->remove(idx);

  if (templateList->isEmpty()) treeWidget->clear();
}

void TemplateDialog::currentItemChanged(QListWidgetItem *itm)
{
  int idx = listWidgetTemplates->row(itm);
  if (idx >= templateList->size()) return;

  Template tpl = templateList->at(idx);
  QStringList lst = tpl.writeToStringList();

  treeWidget->clear();

  QString par, val;
  QStringList::iterator it;
  for (it = lst.begin(); it != lst.end(); ++it) {
    par = *it;
    ++it;
    val = *it;

    QTreeWidgetItem *twitem = new QTreeWidgetItem(treeWidget);
    twitem->setText(0, par);
    twitem->setText(1, val);
  }
}

void TemplateDialog::setTemplates(QVector<Template> *tl)
{
  *templateList = *tl;

  listWidgetTemplates->clear();

  if (templateList->size() == 0) return;

  for (int i = 0; i < templateList->size(); ++i) {
    Template t = templateList->at(i);
    new QListWidgetItem(t.name(), listWidgetTemplates);
  }

  currentItemChanged(listWidgetTemplates->item(0));
}

QVector<Template> * TemplateDialog::getTemplates()
{
  return templateList;
}


