/***************************************************************************
                          texwiz.cpp  -  description
                             -------------------
    begin                : Sam Nov 29 16:47:18 CET 2003
    copyright            : (C) 2003-2008 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "texwiz.h"
#include "ui_texwizmain.h"


#include "previewdialog.h"
#include "headingsdialog.h"
#include "pkgdialog.h"
#include "settingsdialog.h"
#include "ui_titledlg.h"
#include "marginprev.h"
#include "markupdialog.h"
#include "templatedialog.h"
#include "translate.h"


/*
#ifdef USE_KDE
  #include <kapplication.h>
  #include <kfiledialog.h>
  #include <kmessagebox.h>
  #include <dcopclient.h>
#else
  #include <QFileInfo>
  #include <QMessageBox>
#endif
*/

#include <QAction>
#include <QButtonGroup>
#include <QCheckBox>
#include <QCloseEvent>
#include <QComboBox>
#include <QDateTime>
#include <QDateTimeEdit>
#include <QDebug>
#include <QDialog>
#include <QDialogButtonBox>
#include <QFile>
#include <QFileDialog>
#include <QFontMetrics>
#include <QFrame>
#include <QGridLayout>
#include <QIcon>
#include <QInputDialog>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QList>
#include <QListWidget>
#include <QMenu>
#include <QMenuBar>
#include <QMessageBox>
#include <QPixmap>
#include <QPushButton>
#include <QRadioButton>
#include <QRect>
#include <QScrollBar>
#include <QSettings>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QSpinBox>
#include <QSplitter>
#include <QStackedWidget>
#include <QStatusBar>
#include <QString>
#include <QStringList>
#include <QTabWidget>
#include <QTextCodec>
#include <QTextEdit>
#include <QTextStream>
#include <QToolBar>
#include <QVariant>
#include <QWhatsThis>


#define VERSION "2.0.1"

TexwizApp::TexwizApp()
{
  setupUi(this);

  version = VERSION;
  setWindowTitle("TeXwiz " + version);

  templateList = new QVector<Template>();

  koma = false;
  pdf = false;
  twoside = false;
  listSort = true;
  idx = false;
  useKile = false;

  t_head = "";
  t_subject = "";
  t_publisher = "";
  t_dedication = "";
  tb_top = "";
  tb_bottom = "";
  outputFile = "";


  initGui();
  initConnections();
  initListWidget();
  initSettings();

  marginChanged(0);

  // initialize DCOP communication if KDE support is enabled
  #ifdef USE_KDE
    client = kapp->dcopClient();
    bool done = client->attach();
    if (done) qDebug(tr("DCOP communication enabled."));
    else qDebug(tr("DCOP communication disabled."));
  #endif
}

TexwizApp::~TexwizApp()
{
}

// ------------------
// GUI Initialisation
// ------------------

void TexwizApp::initGui()
{
  pushButtonScrhOptions->setEnabled(false); 
  textLabelNote->hide();
  if (!useKile) buttonBoxMain->hide();

  toolBar = new QToolBar(this);
  addToolBar(toolBar);

  QMenu *fileMenu = new QMenu(tr("&File"), this);
  menuBar()->addMenu(fileMenu);

  QMenu *editMenu = new QMenu(tr("&Edit"), this);
  menuBar()->addMenu(editMenu);

  templateMenu = new QMenu(tr("&Templates"), this);
  menuBar()->addMenu(templateMenu);

  QMenu *helpMenu = new QMenu(tr("&Help"), this);
  menuBar()->addMenu(helpMenu);

  QAction *file_preview_act = fileMenu->addAction(QIcon(":/images/view_fit_window.png"), tr("Preview..."));
  QAction *file_save_as_act = fileMenu->addAction(QIcon(":/images/filesaveas.png"), tr("Save As..."));
  QAction *file_exit_act = fileMenu->addAction(QIcon(":/images/exit.png"), tr("Exit"));

  QAction *edit_preferences_act = editMenu->addAction(tr("Preferences..."));

  QAction *templ_save_act = templateMenu->addAction(tr("Save Template"));
  QAction *templ_manage_act = templateMenu->addAction(tr("Manage Templates"));

  QAction *help_whatsthis_act = helpMenu->addAction(QIcon(":/images/contexthelp.png"), tr("What's this?"));
  QAction *help_about_act = helpMenu->addAction(tr("About TeXWiz..."));

  connect(file_save_as_act, SIGNAL(triggered()), this, SLOT(fileSaveAs()));
  connect(file_preview_act, SIGNAL(triggered()), this, SLOT(filePreview()));
  connect(file_exit_act, SIGNAL(triggered()), this, SLOT(close()));
  connect(edit_preferences_act, SIGNAL(triggered()), this, SLOT(preferences()));
  connect(templ_save_act, SIGNAL(triggered()), this, SLOT(saveTemplate()));
  connect(templ_manage_act, SIGNAL(triggered()), this, SLOT(manageTemplates()));
  connect(help_whatsthis_act, SIGNAL(triggered()), this, SLOT(whatsThis()));
  connect(help_about_act, SIGNAL(triggered()), this, SLOT(helpAbout()));

  toolBar->insertAction(0, file_preview_act);
  toolBar->insertAction(0, file_save_as_act);
  toolBar->insertAction(0, help_whatsthis_act);

  labelMarginTop = new QLabel("Top", this);
  labelMarginBottom = new QLabel("Bottom", this);
  labelMarginInner = new QLabel("Inner", this);
  labelMarginOuter = new QLabel("Outer", this);

  labelMarginTop->setAlignment(Qt::AlignRight);
  labelMarginBottom->setAlignment(Qt::AlignRight);
  labelMarginInner->setAlignment(Qt::AlignRight);
  labelMarginOuter->setAlignment(Qt::AlignRight);

  QGridLayout *km_glayout = new QGridLayout(groupBoxKomaMargin);
  km_glayout->setRowStretch(0, 1);
  km_glayout->setRowStretch(5, 1);
  km_glayout->addWidget(new QLabel(tr("Top"), this), 1, 0, 0);
  km_glayout->addWidget(new QLabel(tr("Bottom"), this), 2, 0, 0);
  km_glayout->addWidget(new QLabel(tr("Inner"), this), 3, 0, 0);
  km_glayout->addWidget(new QLabel(tr("Outer"), this), 4, 0, 0);

  km_glayout->addWidget(labelMarginTop, 1, 1, 0);
  km_glayout->addWidget(labelMarginBottom, 2, 1, 0);
  km_glayout->addWidget(labelMarginInner, 3, 1, 0);
  km_glayout->addWidget(labelMarginOuter, 4, 1, 0);

  marginPrev = new MarginPrev(this);
  marginPrev->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  km_glayout->addWidget(marginPrev, 0, 2, 6, 1, 0);
}

void TexwizApp::initConnections()
{
  connect(buttonBoxMain, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBoxMain, SIGNAL(rejected()), this, SLOT(reject()));

  connect(listView, SIGNAL(currentRowChanged(int)), stackedWidget, SLOT(setCurrentIndex(int)));
  connect(spinBoxDiv, SIGNAL(valueChanged(int)), this, SLOT(marginChanged(int)));
  connect(spinBoxBcor, SIGNAL(valueChanged(int)), this, SLOT(marginChanged(int)));
  connect(comboBoxDuplex, SIGNAL(currentIndexChanged(int)), this, SLOT(marginChanged(int)));
  connect(comboBoxPaper, SIGNAL(currentIndexChanged(int)), this, SLOT(marginChanged(int)));

  connect(pushButtonEditMarkup, SIGNAL(clicked()), this, SLOT(editCustomMarkup()));
  connect(pushButtonETitle, SIGNAL(clicked()), this, SLOT(extendedTitles()));
  connect(pushButtonScrhOptions, SIGNAL(clicked()), this, SLOT(headingsOptions()));
  connect(radioButtonETitle, SIGNAL(toggled(bool)), pushButtonETitle, SLOT(setEnabled(bool)));
  connect(checkBoxStaticDate, SIGNAL(toggled(bool)), dateEdit, SLOT(setEnabled(bool)));

  connect(checkBoxKoma, SIGNAL(toggled(bool)), groupBoxToc, SLOT(setEnabled(bool)));
  connect(checkBoxKoma, SIGNAL(toggled(bool)), groupBoxKoma, SLOT(setEnabled(bool)));
  connect(checkBoxKoma, SIGNAL(toggled(bool)), groupBoxKomaMargin, SLOT(setEnabled(bool)));

  connect(buttonBibEdit, SIGNAL(clicked()), this, SLOT(editBibStyles()));
  connect(buttonBibDb, SIGNAL(clicked()), this, SLOT(selectBibDb()));

  connect(buttonPkgEdit, SIGNAL(clicked()), this, SLOT(pkgAdd()));
  connect(buttonPkgIns, SIGNAL(clicked()), this, SLOT(pkgIns()));
  connect(buttonPkgRem, SIGNAL(clicked()), this, SLOT(pkgRem()));
  connect(buttonPkgClear, SIGNAL(clicked()), this, SLOT(pkgClear()));
  connect(buttonPkgSort, SIGNAL(clicked()), this, SLOT(pkgSort()));

  connect(this, SIGNAL(sigPageStyle(int)), this, SLOT(slotShowScrhOptions(int)));
  connect(comboBoxPageStyle, SIGNAL(activated(int)), this, SLOT(slotShowScrhOptions(int)));
}

void TexwizApp::initListWidget()
{
// Widget stack order is:
// Document Class
// Title and Author
// Page Layout
// TOC
// KOMA Script
// Bibliography
// Optional Packages

  // try to get a nice layout of the listview
  QFontMetrics fm = listView->fontMetrics();
  int spacing = 0;
  int w = 0;
  int h = 2 * fm.height();

  // search the largest bounding rect of the text
  w = qMax(fm.width(tr("Title and\nAuthor")), w);
  w = qMax(fm.width(tr("Document\nClass")), w);
  w = qMax(fm.width(tr("Page Layout")), w);
  w = qMax(fm.width(tr("KOMA Script")), w);
  w = qMax(fm.width(tr("Lists")), w);
  w = qMax(fm.width(tr("Bibliography")), w);
  w = qMax(fm.width(tr("Optional\nPackages")), w);

  h += 32 + 5; // height of the icon plus some spacing between text and icon

  listView->setViewMode(QListView::IconMode);
  listView->setIconSize(QSize(32, 32));
  listView->setMovement(QListView::Static);
  listView->setMaximumWidth(w + 2*spacing + 25);
  listView->setMinimumWidth(w + 2*spacing + 25);
  listView->setSpacing(spacing);
  listView->setFlow(QListView::TopToBottom);
  listView->setWrapping(false);

  QListWidgetItem *toButton = new QListWidgetItem(QIcon(":/images/identity.png"), tr("Title and\nAuthor"), listView);
  toButton->setTextAlignment(Qt::AlignHCenter);
  toButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  toButton->setSizeHint(QSize(w, h));

  QListWidgetItem *dcButton = new QListWidgetItem(QIcon(":/images/edit.png"), tr("Document\nClass"), listView);
  dcButton->setTextAlignment(Qt::AlignHCenter);
  dcButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  dcButton->setSizeHint(QSize(w, h));

  QListWidgetItem *plButton = new QListWidgetItem(QIcon(":/images/contents.png"), tr("Page Layout"), listView);
  plButton->setTextAlignment(Qt::AlignHCenter);
  plButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  plButton->setSizeHint(QSize(w, h));

  QListWidgetItem *ksButton = new QListWidgetItem(QIcon(":/images/kontact_journal.png"), tr("KOMA Script"), listView);
  ksButton->setTextAlignment(Qt::AlignHCenter);
  ksButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  ksButton->setSizeHint(QSize(w, h));

  QListWidgetItem *lsButton = new QListWidgetItem(QIcon(":/images/fonts.png"), tr("Lists"), listView);
  lsButton->setTextAlignment(Qt::AlignHCenter);
  lsButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  lsButton->setSizeHint(QSize(w, h));

  QListWidgetItem *biButton = new QListWidgetItem(QIcon(":/images/filefind.png"), tr("Bibliography"), listView);
  biButton->setTextAlignment(Qt::AlignHCenter);
  biButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  biButton->setSizeHint(QSize(w, h));

  QListWidgetItem *opButton = new QListWidgetItem(QIcon(":/images/kthememgr.png"), tr("Optional\nPackages"), listView);
  opButton->setTextAlignment(Qt::AlignHCenter);
  opButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  opButton->setSizeHint(QSize(w, h));


  listView->setCurrentItem(0);
  stackedWidget->setCurrentIndex(0);
}

// -------------------------
// Function implementations:
// -------------------------

QString TexwizApp::collectString()
{
  QString w;

  QString output = QString::null;

  title = textEditTitle->toPlainText();
  author = lineEditAuthor->text();

  // check for KOMA options
  if (checkBoxKoma->isChecked()) koma = true;
  else koma = false;
  if (comboBoxOutputFormat->currentIndex() == 1) pdf = true;
  else pdf = false;
  if (comboBoxDuplex->currentIndex() == 1) twoside = true;
  else twoside = false;
  if (checkBoxIndex->isChecked()) idx = true;
  else idx = false;
  if (checkBoxBiblio->isChecked()) biblio = true;
  else biblio = false;

  output += commentHeader();
  output += documentClass();
  output += collectLang();
  output += usePackages();
  output += pageStyle();
  output += customMacros();
  if (pdf) output += pdfOptions();
  output += pageOptions();

  if (idx) output += "\\makeindex\n\n";

  output += "\\begin{document}\n\n";
  output += collectTitle();
  output += titlePage();
  output += "%--------------- Start of the document --------------\n\n\n\n";
  output += "%---------------- End of the document ---------------\n\n";

  if (biblio) {
    if (radioButtonBibtex->isChecked()) {
      // use bibTeX with database
      output += "\\bibliographystyle{" + comboBoxBibSty->currentText() + "}\n";
      output += tr("% It is strongly recommended to convert the\n"
                   "% following absolute path to a relative path:\n");
      output += "\\bibliography{" + lineEditBibDb->text() + "}\n";
    }
    else {
      // use inline references
      output += "\\begin{thebibliography}{99}\n";
      output += "%  \\bibitem{}\n";
      output += "\\end{thebibliography}\n\n";
    }
  }
  if (idx) output += "\\printindex\n";
  output += "\\end{document}\n";

  return output;
}

QString TexwizApp::collectLang()
{
  collector = QString::null;
  if (comboBoxLang->currentIndex() == 1) collector += "\\usepackage{ngerman}\n";
  if (comboBoxLang->currentIndex() == 2) collector += "\\usepackage[french]{babel}\n";
  if (comboBoxLang->currentIndex() == 3) collector += "\\usepackage[spanish]{babel}\n";

  if (comboBoxEncoding->currentIndex() == 0) collector += "\\usepackage[latin1]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 1) collector += "\\usepackage[latin2]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 2) collector += "\\usepackage[latin3]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 3) collector += "\\usepackage[latin4]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 4) collector += "\\usepackage[latin5]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 5) collector += "\\usepackage[latin9]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 6) 
    collector += "\\usepackage{ucs}  \% (requires 'latex-utf' extension)\n\\usepackage[utf8]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 7) collector += "\\usepackage[ansinew]{inputenc}\n";
  if (comboBoxEncoding->currentIndex() == 8) collector += "\\usepackage[applemac]{inputenc}\n";

  return collector;
}

QString TexwizApp::commentHeader()
{
  collector  = "%----------------------------------------------------\n%\n";
  collector += "% Title:  " + title + "\n";
  collector += "% Author: " + author + "\n";
  collector += "% Date:   ";
  if (comboBoxLang->currentIndex() == 0) collector += dateEdit->date().toString("MMMM d, yyyy");
  if (comboBoxLang->currentIndex() == 1) collector += dateEdit->date().toString("d. MMMM yyyy");
  if (comboBoxLang->currentIndex() == 2) collector += dateEdit->date().toString("d MMMM yyyy").toLower();
  collector += "\n%\n%----------------------------------------------------\n";
  collector += "% Created with TeXwiz " + version + "\n";
  collector += "%----------------------------------------------------\n\n";

  return collector;
}

QString TexwizApp::collectTitle()
{
  collector = "";
  if (!title.isEmpty())  collector = "\n\\title{" + title + "}\n";
  // if maketitle is used, the title tag must be present, also if it's empty
  else {if (!radioButtonNTitle->isChecked()) collector = "\n\\title{}\n";}
  if (!author.isEmpty()) collector += "\\author{" + author + "}\n";
  if (checkBoxStaticDate->isChecked()) {
    if (comboBoxLang->currentIndex() == 0) {
      QString m = Translate::monthName(dateEdit->date().month(), "");
      date = m + dateEdit->date().toString(" d, yyyy");
    }

    if (comboBoxLang->currentIndex() == 1) {
      QString m = Translate::monthName(dateEdit->date().month(), "D");
      date = dateEdit->date().toString("d. ") + m;
      date += dateEdit->date().toString(" yyyy");
    }

    if (comboBoxLang->currentIndex() == 2) {
      QString m = Translate::monthName(dateEdit->date().month(), "F");
      date = dateEdit->date().toString("d ") + m;
      date += dateEdit->date().toString(" yyyy");
    }

    if (comboBoxLang->currentIndex() == 3) {
      QString m = Translate::monthName(dateEdit->date().month(), "E");
      date = dateEdit->date().toString("d ") + "de " + m + " de";
      date += dateEdit->date().toString(" yyyy");
    }
  }
  else date = "\\today";
  collector += "\\date{" + date + "}\n";

  if (koma && (radioButtonETitle->isChecked())) {
    if (!t_extra.isEmpty())      collector += "\\extratitle{" + t_extra + "}\n";
    if (!t_head.isEmpty())       collector += "\\titlehead{" + t_head + "}\n";
    if (!t_subject.isEmpty())    collector += "\\subject{" + t_subject + "}\n";
    if (!t_publisher.isEmpty())  collector += "\\publishers{" + t_publisher + "}\n";
    if (!t_thanks.isEmpty())     collector += "\\thanks{" + t_thanks + "}\n";
    if (!t_dedication.isEmpty()) collector += "\\dedication{" + t_dedication + "}\n";
    if (comboBoxDuplex->currentIndex() == 1) {
      if (!tb_top.isEmpty())     collector += "\\uppertitleback{" + tb_top + "}\n";
      if (!tb_bottom.isEmpty())  collector += "\\lowertitleback{" + tb_bottom + "}\n";
    }
    collector += "\n";
  }

  return collector;
}

QString TexwizApp::documentClass()
{
  collector = "\\documentclass";
  QString options = "[";
  QString w;

  if (comboBoxPaper->currentIndex() == 0) options += "a4paper";
  if (comboBoxPaper->currentIndex() == 1) options += "a5paper";
  if (comboBoxPaper->currentIndex() == 2) options += "b5paper";
  if (comboBoxPaper->currentIndex() == 3) options += "executivepaper";
  if (comboBoxPaper->currentIndex() == 4) options += "legalpaper";
  if (comboBoxPaper->currentIndex() == 5) options += "letterpaper";

  if (comboBoxFontSize->currentIndex() == 0) options += ",\n               10pt";
  if (comboBoxFontSize->currentIndex() == 1) options += ",\n               11pt";
  if (comboBoxFontSize->currentIndex() == 2) options += ",\n               12pt";

  if (comboBoxOutputFormat->currentIndex() == 1) options += ",\n               pdftex";

  if (comboBoxColumns->currentIndex() == 1) options += ",\n               twocolumn";
  if (comboBoxDuplex->currentIndex() == 1) options += ",\n               twoside";
  if (comboBoxOrient->currentIndex() == 1) options += ",\n               landscape";

  if (comboBoxHeadingSize->currentIndex() == 0) options += ",\n               smallheadings";
  if (comboBoxHeadingSize->currentIndex() == 1) options += ",\n               normalheadings";
  if (comboBoxHeadingSize->currentIndex() == 2) options += ",\n               bigheadings";

  if (koma && (checkBoxListsToc->isChecked()))         options += ",\n               liststotoc";
  if (koma && idx && (checkBoxIdxToc->isChecked()))    options += ",\n               idxtotoc";
  if (koma && biblio && (checkBoxBibToc->isChecked())) options += ",\n               bibtotoc";

  if (comboBoxPageStyle->currentIndex() == 4) {
    if (headingOptions[0] == "1") options += ",\n               headsepline";
    if (headingOptions[1] == "1") options += ",\n               footsepline";
  }

  // spezielle KOMA optionen werden angehaengt
  if (koma) {
    options += ",\n               headinclude";
    options += ",\n               footexclude";
    // der DIV-Wert wird auf jeden Fall eingefuegt
    options += ",\n               DIV" + w.setNum(spinBoxDiv->value());
    // der BCOR-Wert wird nur eingefuegt wenn er nicht 0 ist
    if (spinBoxBcor->value()) options += ",\n               BCOR" + w.setNum(spinBoxBcor->value()) + "mm";
    }

  // klammer der optionen schliessen
  options += "\n               ]";

  // falls keine Optionen vorhanden sind sollen keine leeren klammern eingefuegt
  // werden, sondern gar nichts, deshalb wird fuer diesen Fall "options" auf "null"
  // gesetzt  
  if (options == "[]") options = QString::null;

  // verfollstaendige die Zeile mit der Dokumentenklasse
  if (koma) {
    if (comboBoxClass->currentIndex() == 0) collector += options + "{scrartcl}\n\n";
    if (comboBoxClass->currentIndex() == 1) collector += options + "{scrreprt}\n\n";
    if (comboBoxClass->currentIndex() == 2) collector += options + "{scrbook}\n\n";
    }
  else {
    if (comboBoxClass->currentIndex() == 0) collector += options + "{article}\n\n";
    if (comboBoxClass->currentIndex() == 1) collector += options + "{report}\n\n";
    if (comboBoxClass->currentIndex() == 2) collector += options + "{book}\n\n";
    }

  if (comboBoxFontFam->currentIndex() == 1) {
    collector += "\\usepackage{mathptmx}\n"              // times
                 "\\usepackage[scaled=.90]{helvet}\n"
                 "\\usepackage{courier}\n";
  }
  if (comboBoxFontFam->currentIndex() == 2) {
    collector += "\\usepackage{mathpazo}\n"              // palatino
                 "\\usepackage[scaled=.95]{helvet}\n"
                 "\\usepackage{courier}\n"
                 "\\linespread{1.05}\n";
  }
  if (comboBoxFontFam->currentIndex() == 3) collector += "\\usepackage{helvet}\n";
  if (comboBoxFontFam->currentIndex() == 4) collector += "\\usepackage{avant}\n";
  if (comboBoxFontFam->currentIndex() == 5) collector += "\\usepackage{bookman}\n";
  if (comboBoxFontFam->currentIndex() == 6) collector += "\\usepackage{newcent}\n";
  if (idx) collector += "\\usepackage{makeidx}\n";

  // gibt die vollstaendige Zeile zurueck an den Caller
  return collector; 
}

QString TexwizApp::usePackages()
{
  collector = QString::null;
  bool calc = false,
       graph = false,
       space = false;

  for (int i = 0; i < listWidgetPkgLoad->count(); i++) {
    QString s = listWidgetPkgLoad->item(i)->text();
    collector += "\\usepackage{" + s + "}\n";
    if (s == "calc") calc = true;
    if (s == "graphicx") graph = true;
    if (s == "setspace") space = true;
  }

  // these two are always required, thus load if not done yet
  if (!calc) collector += "\\usepackage{calc}\n";
  if (!graph) collector += "\\usepackage{graphicx}\n";

  // handle the setspace packet
  QString lsp;
  if (comboBoxLSpacing->currentIndex() == 0) lsp = "\\singlespacing\n";
  if (comboBoxLSpacing->currentIndex() == 1) lsp = "\\onehalfspacing\n";
  if (comboBoxLSpacing->currentIndex() == 2) lsp = "\\doublespacing\n";

  // if the package setspace was loaded manually,
  // always include the \...spacing line, else only
  // load the package if line spacing is > single
  if (space) collector += lsp;
  else if (comboBoxLSpacing->currentIndex() != 0) {
    collector += "\\usepackage{setspace}\n";
    collector += lsp;
  }

  return collector;
}

QString TexwizApp::pageStyle()
{
  collector = "\n%---------------- Page style options ----------------\n\n";

  // einfache pagesyles brauchen nur eine Zeile...
  if (comboBoxPageStyle->currentIndex() == 0) collector += "\\pagestyle{empty}\n";
  if (comboBoxPageStyle->currentIndex() == 1) collector += "\\pagestyle{plain}\n";
  if (comboBoxPageStyle->currentIndex() == 2) collector += "\\pagestyle{headings}\n";

  // fancy wird gleich speziell formatiert
  if (comboBoxPageStyle->currentIndex() == 3) {
    collector += "\\usepackage{fancyhdr}\n\\pagestyle{fancy}\n";
    collector += "\\renewcommand{\\chaptermark}[1]{\\markboth{\\thechapter\\ #1}{}}\n";
    collector += "\\renewcommand{\\sectionmark}[1]{\\markright{\\thesection\\ #1}{}}\n";
    collector += "\\fancyhf{}\n\\fancyhead[L]{\\leftmark}\n\\fancyhead[R]{\\thepage}\n";
    }

  // fuer scrheadings muss der Options-dialog ausgelesen werden
  if (comboBoxPageStyle->currentIndex() == 4) {
    QString idx2 = "\\pagemark";
    QString idx3 = "\\headmark";
    collector += "\\usepackage[automark]{scrpage2}\n\\pagestyle{scrheadings}\n";
    if (twoside) {
      // header:
      // left even
      if (headingOptions.at(2) == "1") collector += "\\lehead{}\n";
      if (headingOptions.at(2) == "2") collector += "\\lehead{" + idx2 + "}\n";
      if (headingOptions.at(2) == "3") collector += "\\lehead{" + idx3 + "}\n";
      // center even
      if (headingOptions.at(3) == "1") collector += "\\cehead{}\n";
      if (headingOptions.at(3) == "2") collector += "\\cehead{" + idx2 + "}\n";
      if (headingOptions.at(3) == "3") collector += "\\cehead{" + idx3 + "}\n";
      // right even
      if (headingOptions.at(4) == "1") collector += "\\rehead{}\n";
      if (headingOptions.at(4) == "2") collector += "\\rehead{" + idx2 + "}\n";
      if (headingOptions.at(4) == "3") collector += "\\rehead{" + idx3 + "}\n";
      // left odd
      if (headingOptions.at(5) == "1") collector += "\\lohead{}\n";
      if (headingOptions.at(5) == "2") collector += "\\lohead{" + idx2 + "}\n";
      if (headingOptions.at(5) == "3") collector += "\\lohead{" + idx3 + "}\n";
      // center odd
      if (headingOptions.at(6) == "1") collector += "\\cohead{}\n";
      if (headingOptions.at(6) == "2") collector += "\\cohead{" + idx2 + "}\n";
      if (headingOptions.at(6) == "3") collector += "\\cohead{" + idx3 + "}\n";
      // right odd
      if (headingOptions.at(7) == "1") collector += "\\rohead{}\n";
      if (headingOptions.at(7) == "2") collector += "\\rohead{" + idx2 + "}\n";
      if (headingOptions.at(7) == "3") collector += "\\rohead{" + idx3 + "}\n";

      // footer:
      // left even
      if (headingOptions.at(8) == "1") collector += "\\lefoot{}\n";
      if (headingOptions.at(8) == "2") collector += "\\lefoot{" + idx2 + "}\n";
      if (headingOptions.at(8) == "3") collector += "\\lefoot{" + idx3 + "}\n";
      // center even
      if (headingOptions.at(9) == "1") collector += "\\cefoot{}\n";
      if (headingOptions.at(9) == "2") collector += "\\cefoot{" + idx2 + "}\n";
      if (headingOptions.at(9) == "3") collector += "\\cefoot{" + idx3 + "}\n";
      // right even
      if (headingOptions.at(10) == "1") collector += "\\refoot{}\n";
      if (headingOptions.at(10) == "2") collector += "\\refoot{" + idx2 + "}\n";
      if (headingOptions.at(10) == "3") collector += "\\refoot{" + idx3 + "}\n";
      // left odd
      if (headingOptions.at(11) == "1") collector += "\\lofoot{}\n";
      if (headingOptions.at(11) == "2") collector += "\\lofoot{" + idx2 + "}\n";
      if (headingOptions.at(11) == "3") collector += "\\lofoot{" + idx3 + "}\n";
      // center odd
      if (headingOptions.at(12) == "1") collector += "\\cofoot{}\n";
      if (headingOptions.at(12) == "2") collector += "\\cofoot{" + idx2 + "}\n";
      if (headingOptions.at(12) == "3") collector += "\\cofoot{" + idx3 + "}\n";
      // right odd
      if (headingOptions.at(13) == "1") collector += "\\rofoot{}\n";
      if (headingOptions.at(13) == "2") collector += "\\rofoot{" + idx2 + "}\n";
      if (headingOptions.at(13) == "3") collector += "\\rofoot{" + idx3 + "}\n";
    }
    else {
      // header:
      // inner
      if (headingOptions.at(14) == "1") collector += "\\ihead{}\n";
      if (headingOptions.at(14) == "2") collector += "\\ihead{" + idx2 + "}\n";
      if (headingOptions.at(14) == "3") collector += "\\ihead{" + idx3 + "}\n";
      // centre
      if (headingOptions.at(15) == "1") collector += "\\chead{}\n";
      if (headingOptions.at(15) == "2") collector += "\\chead{" + idx2 + "}\n";
      if (headingOptions.at(15) == "3") collector += "\\chead{" + idx3 + "}\n";
      // outer
      if (headingOptions.at(16) == "1") collector += "\\ohead{}\n";
      if (headingOptions.at(16) == "2") collector += "\\ohead{" + idx2 + "}\n";
      if (headingOptions.at(16) == "3") collector += "\\ohead{" + idx3 + "}\n";

      // footer:
      // inner
      if (headingOptions.at(17) == "1") collector += "\\ifoot{}\n";
      if (headingOptions.at(17) == "2") collector += "\\ifoot{" + idx2 + "}\n";
      if (headingOptions.at(17) == "3") collector += "\\ifoot{" + idx3 + "}\n";
      // centre
      if (headingOptions.at(18) == "1") collector += "\\cfoot{}\n";
      if (headingOptions.at(18) == "2") collector += "\\cfoot{" + idx2 + "}\n";
      if (headingOptions.at(18) == "3") collector += "\\cfoot{" + idx3 + "}\n";
      // outer
      if (headingOptions.at(19) == "1") collector += "\\ofoot{}\n";
      if (headingOptions.at(19) == "2") collector += "\\ofoot{" + idx2 + "}\n";
      if (headingOptions.at(19) == "3") collector += "\\ofoot{" + idx3 + "}\n";
    }
  }

  collector += "\n";
  if (koma) {
    if (comboBoxKCaptions->currentIndex() == 0) collector += "\\setcaphanging\n\n";
    if (comboBoxKCaptions->currentIndex() == 1) collector += "\\setcapindent{1em}\n\n";
    if (comboBoxKCaptions->currentIndex() == 2) collector += "\\setcapindent*{1em}\n\n";
    if (comboBoxKCaptions->currentIndex() == 3) collector += "\\setcapindent{-1em}\n\n";
  }
  return collector;
}

QString TexwizApp::pageOptions()
{
  // reset the return string
  collector = "";

  // the usual dummy string
  QString w;
  if (spinBoxTocDepth->value() > 0) 
    collector += "\\setcounter{tocdepth}{" + w.setNum(spinBoxTocDepth->value()) + "}\n";
  if (spinBoxFpf->value() > 0) {
    collector += "\\renewcommand{\\floatpagefraction}{0." + w.setNum(spinBoxFpf->value()) + "}\n";
    collector += "\\renewcommand{\\dblfloatpagefraction}{0." + w.setNum(spinBoxFpf->value()) + "}\n";
    }
  if (checkBoxSloppy->isChecked()) {
    collector += "\n% sloppy wordspacing:\n";
    collector += "\\tolerance 1414\n";
    collector += "\\hbadness 1414\n";
    collector += "\\emergencystretch 1.5em\n";
    collector += "\\hfuzz 0.3pt\n";
    collector += "\\widowpenalty=10000\n";
    collector += "\\vfuzz \\hfuzz\n";
    collector += "\\raggedbottom\n\n";
    }

  return collector;
}

QString TexwizApp::pdfOptions()
{
  collector  = "%---------------- PDF output options ----------------\n\n";
  collector += "\\pdfoutput=1\n\\usepackage{thumbpdf}\n\\usepackage[bookmarks]{hyperref}\n";
  collector += "\\pdfimageresolution=300\n";
  collector += "\\pdfcompresslevel=1\n\n";

  collector += "\\pdfinfo{\n";
  collector += "/Title(" + title + ")\n";
  collector += "/Creator(TeX)\n";
  collector += "/Producer(PdfTeX)\n";
  collector += "/Author(" + author + ")\n";
  collector += "/CreationDate(D:" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + ")\n";
  collector += "/Subject(" + title + ")}\n\n";
  collector += "%----------------------------------------------------\n\n";

  return collector;
}

QString TexwizApp::customMacros()
{
  if (!checkBoxMarkups->isChecked()) return QString::null;
  collector  = "%-------------- Custom Markup Commands --------------\n\n";
  collector += customMarkups.join("\n");
  collector += "\n\n";
  return collector;
}

QString TexwizApp::titlePage()
{
  collector = QString::null;
  if (radioButtonTitle->isChecked() || radioButtonPTitle->isChecked() || radioButtonETitle->isChecked())
    collector += "\\maketitle\n";
  if (radioButtonCTitle->isChecked()) {
    collector += "%----------------- Custom title page ----------------\n\n";
    collector += "\\begin{titlepage}\n"
                 "  \\vspace*{5cm}\n"
                 "  \\Huge\n"
                 "  \\noindent\n"
                 "  \\textsl{\\textbf{" + title + "}}\\\\\n"
                 "  \\rule{\\textwidth}{1.5mm}\n"
                 "  \\begin{flushright}\n"
                 "  \\Large\n"
                 "  \\textsl{\\textbf{" + author + "}}\\\\\n"
                 "  \\large\n"
                 "  \\textsl{" + date + "}\n"
                 "  \\end{flushright}\n"
                 "  \\vfill\n"
                 "  \\normalsize\n"
                 "\\end{titlepage}\n";
    }

  if (radioButtonPTitle->isChecked() || radioButtonCTitle->isChecked()) {
    if (comboBoxClass->currentIndex() == 0) collector += "\\newpage\n";
  }

  if (checkBoxToc->isChecked()) collector += "\\tableofcontents\n\n";
  if (checkBoxLof->isChecked()) collector += "\\listoffigures\n\n";
  if (checkBoxLot->isChecked()) collector += "\\listoftables\n\n";

  return collector;
}

/*
void TexwizApp::setKile(bool b) 
{
  useKile = b;
  if (useKile) {
    fileSaveAsAction->setMenuText(tr("Save and open in Kile..."));
    fileSaveAsAction->setStatusTip(tr("Save and open in Kile..."));
    fileSaveAsAction->setToolTip(tr("Save and open in Kile"));
    MenuBar->hide();
    statusBar()->setShown(false);
    toolBar->setShown(false);
  }
  else {
    frameDialog->hide();
  }
}
*/
// ---------------------
// Slot implementations:
// ---------------------

void TexwizApp::fileSaveAs()
{
  QFileInfo finfo;
  QString filename;

  #ifdef USE_KDE
    filename = KFileDialog::getSaveFileName(workingDir, "*.tex|LaTeX source file (*.tex)", this);
  #else
    filename = QFileDialog::getSaveFileName(this, tr("Save File As..."), workingDir, "*.tex *.TEX");
  #endif

  if (filename.isEmpty()) return;
  finfo.setFile(filename);

  // if no extension ".tex" is added, TeXwiz will add it:
  if (finfo.suffix().toLower() != "tex") filename += ".tex";
  finfo.setFile(filename);
  QFile file(filename);
  int cd = comboBoxEncoding->currentIndex();

  QTextCodec *codec = QTextCodec::codecForLocale();

  if (cd == 0) codec = QTextCodec::codecForName("Latin1");
  if (cd == 1) codec = QTextCodec::codecForName("ISO8859-2");
  if (cd == 2) codec = QTextCodec::codecForName("ISO8859-3");
  if (cd == 3) codec = QTextCodec::codecForName("ISO8859-4");
  if (cd == 4) codec = QTextCodec::codecForName("ISO8859-5");
  if (cd == 5) codec = QTextCodec::codecForName("ISO8859-15");
  if (cd == 6) codec = QTextCodec::codecForName("UTF-8");
  if (cd == 7) codec = QTextCodec::codecForName("CP1252");
  if (cd == 8) codec = QTextCodec::codecForName("Apple Roman");

  bool _kile = false;
  #ifdef USE_KDE
    if (useKile) _kile = true;
  #endif

  // in kile mode
  if (_kile) {
    #ifdef USE_KDE  // that's necessary again, because some variables are defined only in this mode
    file.setName(filename);

    if (!file.open(IO_WriteOnly)) {
      KMessageBox::error(0, tr("Couldn't save file\n") + filename, tr("Save failed"));
      return;
    }


    QTextStream s(&file);
    s.setCodec(codec);
    s << collectString();
    file.close();

    // if using KDE, open the file in kile
    client->send("kile",
                 "Kile",
                 "fileSelected(QString)",
                 filename);
    #endif
  }
  else {
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
      #ifdef USE_KDE
        KMessageBox::error(this, tr("Couldn't save file\n") + filename, tr("Save failed"));
      #else
        QMessageBox::warning(this, tr("Save failed"), tr("Couldn't save file\n") + filename);
      #endif
      return;
    }

    QTextStream s(&file);
    s.setCodec(codec);
    s << collectString();
    file.close();
  }

  if (_kile) close();
}

void TexwizApp::filePreview()
{
  PreviewDialog *prev = new PreviewDialog();
  prev->setFont(prevFont);
  prev->setText(collectString());
  if (restoreSize) prev->resize(prev_w, prev_h);
  if (restorePos) prev->move(prev_x, prev_y);

  // the dialog can only be closed by rejecting it, thus settings have to
  // be saved in this case
  if (prev->exec() == QDialog::Rejected) {
    prevFont = prev->getFont();
    prev_w = prev->width();
    prev_h = prev->height();
    prev_x = prev->x();
    prev_y = prev->y();
  }

  delete prev;
}

void TexwizApp::resizeEvent(QResizeEvent *)
{
  emit sigResize();
}

void TexwizApp::showEvent(QShowEvent *)
{
  emit sigResize();
}

void TexwizApp::closeEvent(QCloseEvent *ce)
{
  saveSettings();
  ce->accept();

  return;
}

void TexwizApp::accept()
{

}

void TexwizApp::reject()
{

}

void TexwizApp::helpAbout()
{
  #ifdef USE_KDE
    KMessageBox::about(this,
                       tr("TeXwiz Version " + version + "\n(c) 2003-2007 by Nicola Doebelin"),
		       tr("About...") );
  #else
    QMessageBox::about(this, tr("About..."),
                       tr("TeXwiz\nVersion ") + version + tr("\n(c) 2003-2007 by Nicola Doebelin"));
  #endif
}

void TexwizApp::toggleKoma(bool b)
{
  groupBoxKoma->setEnabled(b);
  groupBoxKomaMargin->setEnabled(b);

  if (b) {
    if (radioButtonETitle->isChecked()) pushButtonETitle->setEnabled(b);
  } else  {
    pushButtonETitle->setEnabled(b);
    if (radioButtonETitle->isChecked()) radioButtonNTitle->setChecked(true);
  }
  radioButtonETitle->setEnabled(b);

  groupBoxToc->setEnabled(b);
}

void TexwizApp::extendedTitles()
{
  QDialog *tdlg = new QDialog();
  Ui::TitleDlg ui;
  ui.setupUi(tdlg);

  ui.editTitleHead->setText(t_head);
  ui.editSubject->setText(t_subject);
  ui.editPublisher->setText(t_publisher);
  ui.editThanks->setText(t_thanks);
  ui.editDedication->setText(t_dedication);
  ui.editExtra->setText(t_extra);
  ui.editBsTop->setText(tb_top);
  ui.editBsBottom->setText(tb_bottom);

  if (comboBoxDuplex->currentIndex() == 0) ui.tabWidget->setTabEnabled(2, false);
  else ui.tabWidget->setTabEnabled(2, true);

  if (tdlg->exec() == QDialog::Accepted) {
    t_head       = ui.editTitleHead->toPlainText();
    t_subject    = ui.editSubject->toPlainText();
    t_publisher  = ui.editPublisher->toPlainText();
    t_thanks     = ui.editThanks->toPlainText();
    t_dedication = ui.editDedication->toPlainText();
    t_extra      = ui.editExtra->toPlainText();
    tb_top       = ui.editBsTop->toPlainText();
    tb_bottom    = ui.editBsBottom->toPlainText();
  }

  delete tdlg;
}

void TexwizApp::editCustomMarkup()
{
  MarkupDialog *mdlg = new MarkupDialog();
  mdlg->setList(customMarkups);

  if (mdlg->exec() == QDialog::Accepted) customMarkups = mdlg->getList();

  delete mdlg;
}

void TexwizApp::slotShowScrhOptions(int a)
{
  if (a == 4) {
    pushButtonScrhOptions->setEnabled(true);
    textLabelNote->show();
    }
  else {
    pushButtonScrhOptions->setEnabled(false);
    textLabelNote->hide();
    }
}

void TexwizApp::headingsOptions()
{
 HeadingsDialog *hdlg = new HeadingsDialog();

  if (comboBoxDuplex->currentIndex() == 0) hdlg->setTwoside(false);
  else hdlg->setTwoside(true);
  hdlg->setOptions(headingOptions);

  if (hdlg->exec() == QDialog::Accepted) {
    headingOptions = hdlg->getOptions();
  }
  delete hdlg;
}

void TexwizApp::marginChanged(int)
{
  QString s;
  float div = (float)spinBoxDiv->value();
  float bcor = (float)spinBoxBcor->value();
  float w = 210.0, h = 297.0;
  float i = 0.0, o = 0.0, t = 0.0, b = 0.0;
  if (comboBoxPaper->currentIndex() == 0) {w = 210.0;  h = 297.0;}
  if (comboBoxPaper->currentIndex() == 1) {w = 148.0;  h = 210.0;}
  if (comboBoxPaper->currentIndex() == 2) {w = 176.0;  h = 250.0;}
  if (comboBoxPaper->currentIndex() == 3) {w = 184.15; h = 266.7;}
  if (comboBoxPaper->currentIndex() == 4) {w = 215.9;  h = 355.6;}
  if (comboBoxPaper->currentIndex() == 5) {w = 215.9;  h = 279.4;}

  //check for landscape and swap w and h if necessary
  if (comboBoxOrient->currentIndex() == 1) {
    float x = w;
    w = h;
    h = x;
  }

  if (comboBoxDuplex->currentIndex() == 0) {
    i = bcor + 1.5 * ((w - bcor) / div);
    o = 1.5 * (w - bcor) / div;
    marginPrev->setTwoside(false);
  }
  else {
    i = bcor + 1.0 * ((w - bcor) / div);
    o = 2.0 * (w - bcor) / div;
    marginPrev->setTwoside(true);
  }

  t = h / div;
  b = 2.0 * h / div;

  marginPrev->setMargin(w, h, div, bcor);

  labelMarginTop->setText(s.sprintf("%3.1f mm", t));
  labelMarginBottom->setText(s.sprintf("%3.1f mm", b));
  labelMarginInner->setText(s.sprintf("%3.1f mm", i));
  labelMarginOuter->setText(s.sprintf("%3.1f mm", o));  
}

void TexwizApp::pkgIns()
{
  QList<QListWidgetItem *> lst = listWidgetPkgAvail->selectedItems();

  while (!lst.isEmpty()) {
    QListWidgetItem *itm = lst.takeFirst();
    new QListWidgetItem(itm->text(), listWidgetPkgLoad);
    delete itm;
  }
}

void TexwizApp::pkgRem()
{
  QList<QListWidgetItem *> lst = listWidgetPkgLoad->selectedItems();

  while (!lst.isEmpty()) {
    QListWidgetItem *itm = lst.takeFirst();
    new QListWidgetItem(itm->text(), listWidgetPkgAvail);
    delete itm;
  }
}

void TexwizApp::pkgAdd()
{
  QStringList l;
  for (int i = 0; i < listWidgetPkgAvail->count(); i++) {
    l.push_back(listWidgetPkgAvail->item(i)->text());
  }

  for (int i = 0; i < listWidgetPkgLoad->count(); i++) {
    l.push_back(listWidgetPkgLoad->item(i)->text());
  }

  PkgDialog *pd = new PkgDialog();
  pd->setTitle(tr("Add Package(s)"));
  pd->setLabel(tr("Package Name"));
  pd->setDirs(texDir, ltexDir);
  pd->setSuffix("sty");
  pd->setItems(l);

  if (pd->exec() == QDialog::Accepted) {
    listWidgetPkgAvail->clear();
    listWidgetPkgLoad->clear();
    listWidgetPkgAvail->addItems(pd->getItems());
  }

  delete pd;
}

void TexwizApp::pkgClear()
{
  while (listWidgetPkgLoad->count()) {
    QListWidgetItem *it = listWidgetPkgLoad->takeItem(0);
    listWidgetPkgAvail->addItem(it);
  }
}

void TexwizApp::pkgSort()
{
  if (listSort) {
    listWidgetPkgAvail->sortItems(Qt::AscendingOrder);
    listWidgetPkgLoad->sortItems(Qt::AscendingOrder);
  } else {
    listWidgetPkgAvail->sortItems(Qt::DescendingOrder);
    listWidgetPkgLoad->sortItems(Qt::DescendingOrder);
  }

  listSort = !listSort;
}

/*
void TexwizApp::toggleToolbar(bool b) {toolBar->setShown(b);}
void TexwizApp::toggleStatusbar(bool b) {statusBar()->setShown(b);}
*/

void TexwizApp::initSettings()
{
  QSettings settings;
  QString avail, loaded, font, bibstyles;

  settings.beginGroup("/texwiz");
    settings.beginGroup("/geometry");
      int w = settings.value("/width", 500).toInt();
      int h = settings.value("/height", 300).toInt();
      int x = settings.value("/x_pos", 0).toInt();
      int y = settings.value("/y_pos", 0).toInt();
      #ifdef Q_OS_MACX
        if (y < 10) y = 10;
      #endif
      prev_w = settings.value("/preview_width", 400).toInt();
      prev_h = settings.value("/preview_height", 300).toInt();
      prev_x = settings.value("/preview_x", 100).toInt();
      prev_y = settings.value("/preview_y", 100).toInt();
    settings.endGroup();
    settings.beginGroup("/preferences");
      font = settings.value("/font_prevdlg", "Courier [Adobe],10,-1,5,50,0,0,0,0,0").toString();
      workingDir = settings.value("/workingdir", "").toString();
      texDir = settings.value("/texdir", "").toString();
      ltexDir = settings.value("/localtexdir", "").toString();
      lineEditBibDb->setText(settings.value("/bibtex_db", "").toString());
      bool sb = settings.value("/statusbar", true).toBool();
      bool tb = settings.value("/toolbar", true).toBool();
      restoreSize = settings.value("/restore_Size", true).toBool();
      restorePos = settings.value("/restore_Pos", true).toBool();
      oldTemplates = settings.value("/template_names").toStringList();
    settings.endGroup();
    settings.beginGroup("/tex_options");
      checkBoxStaticDate->setChecked(settings.value("/staticDate", false).toBool());
      int titleButton = settings.value("/id_buttonTitle", 0).toInt();
      comboBoxEncoding->setCurrentIndex(settings.value("/encoding", 0).toInt());
      checkBoxToc->setChecked(settings.value("/toc", false).toBool());
      groupBoxToc->setEnabled(settings.value("/toc", false).toBool());
      checkBoxLof->setChecked(settings.value("/lof", false).toBool());
      checkBoxLot->setChecked(settings.value("/lot", false).toBool());
      checkBoxIndex->setChecked(settings.value("/idx", false).toBool());
      checkBoxBiblio->setChecked(settings.value("/bib", false).toBool());
      comboBoxClass->setCurrentIndex(settings.value("/dclass", 0).toInt());
      comboBoxPaper->setCurrentIndex(settings.value("/paper", 0).toInt());
      comboBoxFontSize->setCurrentIndex(settings.value("/fontsize", 0).toInt());
      comboBoxFontFam->setCurrentIndex(settings.value("/fontfam", 0).toInt());
      comboBoxOutputFormat->setCurrentIndex(settings.value("/format", 0).toInt());
      comboBoxLang->setCurrentIndex(settings.value("/language", 0).toInt());
      bool use_koma = settings.value("/use_koma", true).toBool();
      comboBoxDuplex->setCurrentIndex(settings.value("/duplex", 0).toInt());
      comboBoxColumns->setCurrentIndex(settings.value("/columns", 0).toInt());
      comboBoxOrient->setCurrentIndex(settings.value("/orientation", 0).toInt());
      comboBoxPageStyle->setCurrentIndex(settings.value("/pagestyle", 0).toInt());
      int bibTexOpt = settings.value("/bib_inline", 0).toInt();
      spinBoxTocDepth->setValue(settings.value("/toc_depth", 0).toInt());
      checkBoxListsToc->setChecked(settings.value("/lists_to_toc", false).toBool());
      checkBoxIdxToc->setChecked(settings.value("/idx_to_toc", false).toBool());
      checkBoxBibToc->setChecked(settings.value("/bib_to_toc", false).toBool());
      spinBoxFpf->setValue(settings.value("/float_page_fraction", 0).toInt());
      checkBoxSloppy->setChecked(settings.value("/sloppy", false).toBool());
      comboBoxLSpacing->setCurrentIndex(settings.value("/lspacing", 0).toInt());
      comboBoxKCaptions->setCurrentIndex(settings.value("/captions", 0).toInt());
      checkBoxMarkups->setChecked(settings.value("/use_custom_macros", false).toBool());
      headingOptions = settings.value("/heading_options").toStringList();
      customMarkups = settings.value("/custom_macros").toStringList();
    settings.endGroup();

    settings.beginGroup("/contents");
      title = settings.value("/title", "").toString();
      author = settings.value("/author", "").toString();

      textEditTitle->setText(title);
      lineEditAuthor->setText(author);

      QString date_string = settings.value("/staticDateValue","").toString();
      if ((checkBoxStaticDate->isChecked()) && !date_string.isEmpty()) {
        dateEdit->setDate(QDate::fromString(date_string, QString("yyyymmdd")));
      } else {
        dateEdit->setDate(QDate::currentDate());
      }

      t_extra = settings.value("/t_extra", "").toString();
      t_head = settings.value("/t_head", "").toString();
      t_subject = settings.value("/t_subject", "").toString();
      t_publisher = settings.value("/t_publisher", "").toString();
      t_thanks = settings.value("/t_thanks", "").toString();
      t_dedication = settings.value("/t_dedication", "").toString();
      tb_top = settings.value("/tb_top", "").toString();
      tb_bottom = settings.value("/tb_bottom", "").toString();
    settings.endGroup();

    settings.beginGroup("/packages");
      avail = settings.value("/pkgs", "").toString();
      loaded = settings.value("/pkgs_loaded", "").toString();
      bibstyles = settings.value("/bib_styles", "").toString();
      int bs = settings.value("/bib_current_style", 0).toInt();
    settings.endGroup();

    settings.beginGroup("/templates");
      templateList->clear();
      QStringList entry;
      QStringList::iterator it = oldTemplates.begin();

      Template t;
      while (it != oldTemplates.end()) {
        entry = settings.value(*it).toStringList();
        t.setFromLongStringList(entry);
        templateList->push_back(t);
        ++it;
      }
    settings.endGroup();
  settings.endGroup();

  if (restoreSize) resize(w, h);
  if (restorePos) move(x, y);

  checkBoxKoma->setChecked(use_koma);
  toggleKoma(use_koma);

  while (headingOptions.size() < 20) headingOptions.push_back("0");

  listWidgetPkgAvail->clear();
  listWidgetPkgLoad->clear();
  comboBoxBibSty->clear();
  listWidgetPkgAvail->addItems(avail.split(";", QString::SkipEmptyParts));
  listWidgetPkgLoad->addItems(loaded.split(";", QString::SkipEmptyParts));
  comboBoxBibSty->insertItems(-1, bibstyles.split(";", QString::SkipEmptyParts));
//   groupBoxBibOpt->setEnabled(checkBoxBiblio->isChecked());
  groupBoxBibtex->setEnabled(radioButtonBibtex->isChecked());

  if (comboBoxDuplex->currentIndex() == 0) marginPrev->setTwoside(false);
  if (comboBoxDuplex->currentIndex() == 1) marginPrev->setTwoside(true);

  comboBoxBibSty->setCurrentIndex(bs);
  prevFont.fromString(font);

  switch (titleButton) {
    case 0: radioButtonNTitle->setChecked(true); break;
    case 1: radioButtonTitle->setChecked(true);  break;
    case 2: radioButtonPTitle->setChecked(true); break;
    case 3: radioButtonCTitle->setChecked(true); break;
    case 4: radioButtonETitle->setChecked(true); break;
  }

  switch (bibTexOpt) {
    case 0: radioButtonBibtex->setChecked(true); break;
    case 1: radioButtonThebib->setChecked(true); break;
  }

//   statusbarAction->setOn(sb);
//   toolbarAction->setOn(tb);
//   statusBar()->setShown(sb);
//   toolBar->setShown(tb);
  updateTemplates();
}

void TexwizApp::saveSettings(bool all)
{
  QString avail, bibstyles, loaded;

  for (int i = 0; i < listWidgetPkgAvail->count(); i++)
    avail += listWidgetPkgAvail->item(i)->text() + ";";
  for (int i = 0; i < listWidgetPkgLoad->count(); i++)
    loaded += listWidgetPkgLoad->item(i)->text() + ";";
  for (int i = 0; i < comboBoxBibSty->count(); i++)
    bibstyles += comboBoxBibSty->itemText(i) + ";";

  QStringList tmpl;
  for (int i = 0; i < templateList->size(); ++i) {
    Template t = templateList->at(i);
    tmpl.append("/" + t.name());
  }

  QSettings settings;
  settings.beginGroup("/texwiz");
    settings.beginGroup("/geometry");
      settings.setValue("/width", width());
      settings.setValue("/height", height());
      settings.setValue("/x_pos", x());
      settings.setValue("/y_pos", y());
      settings.setValue("/preview_width", prev_w);
      settings.setValue("/preview_height", prev_h);
      settings.setValue("/preview_x", prev_x);
      settings.setValue("/preview_y", prev_y);
    settings.endGroup();
    settings.beginGroup("/preferences");
      settings.setValue("/font_prevdlg", prevFont.toString());
      settings.setValue("/workingdir", workingDir);
      settings.setValue("/texdir", texDir);
      settings.setValue("/localtexdir", ltexDir);
      settings.setValue("/bibtex_db", lineEditBibDb->text());
//     if (!useKile) {
//       settings.setValue("/statusbar", statusBar()->isShown());
//       settings.setValue("/toolbar", toolBar->isShown());
//     }
      settings.setValue("/restore_Size", restoreSize);
      settings.setValue("/restore_Pos", restorePos);
      settings.setValue("/template_names", tmpl);
    settings.endGroup();
    settings.beginGroup("/tex_options");
      if (all) {
        int titleValue = 0;
        if (radioButtonTitle->isChecked())  titleValue = 1;
        if (radioButtonPTitle->isChecked()) titleValue = 2;
        if (radioButtonCTitle->isChecked()) titleValue = 3;
        if (radioButtonETitle->isChecked()) titleValue = 4;
        settings.setValue("/id_buttonTitle", titleValue);

        int bibtexValue = 0;
        if (radioButtonThebib->isChecked()) bibtexValue = 1;
        settings.setValue("/bib_inline", bibtexValue);

        settings.setValue("/staticDate", checkBoxStaticDate->isChecked());
        settings.setValue("/encoding", comboBoxEncoding->currentIndex());
        settings.setValue("/toc", checkBoxToc->isChecked());
        settings.setValue("/lof", checkBoxLof->isChecked());
        settings.setValue("/lot", checkBoxLot->isChecked());
        settings.setValue("/idx", checkBoxIndex->isChecked());
        settings.setValue("/bib", checkBoxBiblio->isChecked());
        settings.setValue("/dclass", comboBoxClass->currentIndex());
        settings.setValue("/paper", comboBoxPaper->currentIndex());
        settings.setValue("/fontsize", comboBoxFontSize->currentIndex());
        settings.setValue("/fontfam", comboBoxFontFam->currentIndex());
        settings.setValue("/format", comboBoxOutputFormat->currentIndex());
        settings.setValue("/language", comboBoxLang->currentIndex());
        settings.setValue("/use_koma", checkBoxKoma->isChecked());
        settings.setValue("/duplex", comboBoxDuplex->currentIndex());
        settings.setValue("/columns", comboBoxColumns->currentIndex());
        settings.setValue("/orientation", comboBoxOrient->currentIndex());
        settings.setValue("/pagestyle", comboBoxPageStyle->currentIndex());
        settings.setValue("/toc_depth", spinBoxTocDepth->value());
        settings.setValue("/lists_to_toc", checkBoxListsToc->isChecked());
        settings.setValue("/idx_to_toc", checkBoxIdxToc->isChecked());
        settings.setValue("/bib_to_toc", checkBoxBibToc->isChecked());
        settings.setValue("/float_page_fraction", spinBoxFpf->value());
        settings.setValue("/sloppy", checkBoxSloppy->isChecked());
        settings.setValue("/lspacing", comboBoxLSpacing->currentIndex());
        settings.setValue("/captions", comboBoxKCaptions->currentIndex());
        settings.setValue("/use_custom_macros", checkBoxMarkups->isChecked());
        settings.setValue("/heading_options", headingOptions);
      }
      settings.setValue("/custom_macros", customMarkups);
    settings.endGroup();

    settings.beginGroup("/contents");
      if (all) {
        settings.setValue("/title", textEditTitle->toPlainText());
        settings.setValue("/author", lineEditAuthor->text());
        settings.setValue("/staticDateValue", dateEdit->date().toString(QString("yyyymmdd")));
        settings.setValue("/t_extra", t_extra);
        settings.setValue("/t_head", t_head);
        settings.setValue("/t_subject", t_subject);
        settings.setValue("/t_publisher", t_publisher);
        settings.setValue("/t_thanks", t_thanks);
        settings.setValue("/t_dedication", t_dedication);
        settings.setValue("/tb_top", tb_top);
        settings.setValue("/tb_bottom", tb_bottom);
      }
    settings.endGroup();

    settings.beginGroup("/packages");
      settings.setValue("/pkgs", avail);
      settings.setValue("/pkgs_loaded", loaded);
      settings.setValue("/bib_styles", bibstyles);
      settings.setValue("/bib_current_style", comboBoxBibSty->currentIndex());
    settings.endGroup();
    settings.beginGroup("/templates");
      // remove old template entries, otherwise they may accumulate when renamed
      QStringList::iterator it = oldTemplates.begin();
      while (it != oldTemplates.end()) {
        settings.remove(*it);
        it++;
      }

      // write new templates
      Template tp;
      for (int i = 0; i < templateList->size(); ++i) {
        tp = templateList->at(i);
        QString lname = "/" + tp.name();
        settings.setValue(lname, tp.writeToStringList());
      }
    settings.endGroup();
  settings.endGroup();
}

void TexwizApp::saveAllSettings()
{
  saveSettings(true);
}

void TexwizApp::preferences()
{
  SettingsDialog *sd = new SettingsDialog();
  connect(sd, SIGNAL(saveAll()), this, SLOT(saveAllSettings()));

  sd->setWd(workingDir);
  sd->setTd(texDir);
  sd->setLtd(ltexDir);
  sd->setRestoreSize(restoreSize);
  sd->setRestorePos(restorePos);

  if (sd->exec() == QDialog::Accepted) {
    workingDir = sd->getWd();
    texDir = sd->getTd();
    ltexDir = sd->getLtd();
    restoreSize = sd->getRestoreSize();
    restorePos = sd->getRestorePos();
  }

  delete sd;
}

void TexwizApp::selectBibDb()
{
  QString bibdb = "";
  #ifdef USE_KDE
    bibdb = KFileDialog::getOpenFileName(workingDir,
                                         "BibTeX (*.bib *.BIB|BibTeX Database (*.bib))",
                                         this,
                                         tr("Load BibTeX Database"));
  #else
    bibdb = QFileDialog::getOpenFileName(this,
                                         tr("Select a BibTeX database"),
                                         workingDir,
                                         "BibTeX (*.bib *.BIB)");
  #endif
  if (!bibdb.isEmpty()) lineEditBibDb->setText(bibdb);
}

void TexwizApp::editBibStyles()
{
  QStringList l;
  for (int i = 0; i < comboBoxBibSty->count(); i++) {
    l.push_back(comboBoxBibSty->itemText(i));
  }

  PkgDialog *pd = new PkgDialog();
  pd->setTitle(tr("Edit BibTeX Styles"));
  pd->setLabel(tr("Style Name"));
  pd->setDirs(texDir, ltexDir);
  pd->setSuffix("bst");
  pd->setItems(l);

  if (pd->exec() == QDialog::Accepted) {
    comboBoxBibSty->clear();
    comboBoxBibSty->addItems(pd->getItems());
  }

  delete pd;
}

void TexwizApp::whatsThis()
{
  QWhatsThis::enterWhatsThisMode();
}

void TexwizApp::saveTemplate()
{
  Template t;
  bool ok;
  QString name = QInputDialog::getText(this,
                                       tr("New Template"),
                                       tr("Name of the Template:"),
                                       QLineEdit::Normal,
                                       QString::null, &ok);

  if (!ok || name.isEmpty()) {
    return;
  }

  t.setName(name);

  t.setTitle(textEditTitle->toPlainText(),
              lineEditAuthor->text(),
              dateEdit->date(),
              checkBoxStaticDate->isChecked());

  t.setDocClass(comboBoxClass->currentIndex(),
                 comboBoxPaper->currentIndex(),
                 comboBoxFontSize->currentIndex(),
                 comboBoxFontFam->currentIndex(),
                 comboBoxOutputFormat->currentIndex(),
                 comboBoxEncoding->currentIndex(),
                 comboBoxLang->currentIndex(),
                 checkBoxKoma->isChecked());

  t.setCustomMarkup(checkBoxMarkups->isChecked(), customMarkups);

  int bTitle = 0;
  if (radioButtonTitle->isChecked()) bTitle = 1;
  if (radioButtonPTitle->isChecked()) bTitle = 2;
  if (radioButtonCTitle->isChecked()) bTitle = 3;
  if (radioButtonETitle->isChecked()) bTitle = 4;


  t.setTitlePage(bTitle, t_head, t_subject,
                  t_publisher, t_thanks, t_extra, t_dedication,
                  tb_top, tb_bottom);

  t.setPageLayout(comboBoxDuplex->currentIndex(),
                   comboBoxColumns->currentIndex(),
                   comboBoxOrient->currentIndex(),
                   comboBoxLSpacing->currentIndex(),
                   comboBoxPageStyle->currentIndex(),
                   spinBoxFpf->value(),
                   checkBoxSloppy->isChecked());

  t.setLists(checkBoxToc->isChecked(),
              checkBoxLof->isChecked(),
              checkBoxLot->isChecked(),
              checkBoxIndex->isChecked(),
              checkBoxBiblio->isChecked());

  t.setListsToc(checkBoxListsToc->isChecked(),
                 checkBoxIdxToc->isChecked(),
                 checkBoxBibToc->isChecked(),
                 spinBoxTocDepth->value());

  t.setKoma(spinBoxDiv->value(),
             spinBoxBcor->value(),
             comboBoxHeadingSize->currentIndex(),
             comboBoxKCaptions->currentIndex());

  QStringList lsa, lsb;
  for (int i = 0; i < comboBoxBibSty->count(); i++) lsa.append(comboBoxBibSty->itemText(i));

  int bBibOpt = 0;
  if (radioButtonThebib->isChecked()) bBibOpt = 1;

  t.setBibtex(bBibOpt,
               comboBoxBibSty->currentIndex(),
               lineEditBibDb->text(),
               lsa);

  lsa.clear();
  for (int i = 0; i < listWidgetPkgAvail->count(); i++) {
    lsa.append(listWidgetPkgAvail->item(i)->text());
  }

  for (int i = 0; i < listWidgetPkgLoad->count(); i++) {
    lsb.append(listWidgetPkgLoad->item(i)->text());
  }

  t.setPackages(lsa, lsb);

  templateList->append(t);
  updateTemplates();
}

void TexwizApp::manageTemplates()
{
  TemplateDialog *tpdlg = new TemplateDialog();
  tpdlg->setTemplates(templateList);

  if (tpdlg->exec() == QDialog::Accepted) {
    templateList = tpdlg->getTemplates();
  }

  delete tpdlg;

  updateTemplates();
}

void TexwizApp::setTemplate(int i)
{
  if (i >= templateList->size()) return;
  if (i < 0) return;

  Template t = templateList->at(i);

  textEditTitle->setText(t.title());
  lineEditAuthor->setText(t.author());
  dateEdit->setDate(t.date());
  checkBoxStaticDate->setChecked(t.staticDate());

  comboBoxClass->setCurrentIndex(t.documentClass());
  comboBoxPaper->setCurrentIndex(t.paperSize());
  comboBoxFontSize->setCurrentIndex(t.fontSize());
  comboBoxFontFam->setCurrentIndex(t.fontFam());
  comboBoxOutputFormat->setCurrentIndex(t.outputFormat());
  comboBoxEncoding->setCurrentIndex(t.encoding());
  comboBoxLang->setCurrentIndex(t.language());
  checkBoxKoma->setChecked(t.koma());

  checkBoxMarkups->setChecked(t.useCustomMarkup());

  radioButtonNTitle->setChecked(false);
  radioButtonTitle->setChecked(false);
  radioButtonPTitle->setChecked(false);
  radioButtonCTitle->setChecked(false);
  radioButtonETitle->setChecked(false);

  switch (t.titlePageButton())
  {
    case 0: radioButtonNTitle->setChecked(true); break;
    case 1: radioButtonTitle->setChecked(true); break;
    case 2: radioButtonPTitle->setChecked(true); break;
    case 3: radioButtonCTitle->setChecked(true); break;
    case 4: radioButtonETitle->setChecked(true); break;
  }

  t_head = t.titleHead();
  t_subject = t.subject();
  t_publisher = t.publisher();
  t_thanks = t.thanks();
  t_extra = t.extra();
  t_dedication = t.dedication();
  tb_top = t.backsideTop();
  tb_bottom = t.backsideBottom();

  comboBoxDuplex->setCurrentIndex(t.duplex());
  comboBoxColumns->setCurrentIndex(t.columns());
  comboBoxOrient->setCurrentIndex(t.orientation());
  comboBoxLSpacing->setCurrentIndex(t.lineSpacing());
  comboBoxPageStyle->setCurrentIndex(t.pageStyle());
  spinBoxFpf->setValue(t.floatPageFraction());
  checkBoxSloppy->setChecked(t.sloppy());

  checkBoxToc->setChecked(t.toc());
  checkBoxLof->setChecked(t.lof());
  checkBoxLot->setChecked(t.lot());
  checkBoxIndex->setChecked(t.index());
  checkBoxBiblio->setChecked(t.bibliography());

  checkBoxListsToc->setChecked(t.lofToToc());
  checkBoxIdxToc->setChecked(t.idxToToc());
  checkBoxBibToc->setChecked(t.bibToToc());
  spinBoxTocDepth->setValue(t.tocDepth());

  spinBoxDiv->setValue(t.div());
  spinBoxBcor->setValue(t.bcor());
  comboBoxHeadingSize->setCurrentIndex(t.headingSize());
  comboBoxKCaptions->setCurrentIndex(t.captions());

  comboBoxBibSty->clear();
  comboBoxBibSty->addItems(t.bibtexStylesList());

  radioButtonBibtex->setChecked(false);
  radioButtonThebib->setChecked(false);

  switch (t.bibtexType())
  {
    case 0: radioButtonBibtex->setChecked(true); break;
    case 1: radioButtonThebib->setChecked(true); break;
  }

  comboBoxBibSty->setCurrentIndex(t.bibtexStyle());
  lineEditBibDb->setText(t.bibtexDatabase());

  listWidgetPkgAvail->clear();
  listWidgetPkgLoad->clear();
  listWidgetPkgAvail->addItems(t.packagesAvailable());
  listWidgetPkgLoad->addItems(t.packagesLoaded());
}

void TexwizApp::updateTemplates()
{
  QList<QAction *> actions = templateMenu->actions();

  while (actions.size() > 2) {
    delete actions.takeLast();
  }

  for (int i = 0; i < templateList->size(); ++i) {
    Template tmpl = templateList->at(i);
    QAction *act = templateMenu->addAction(tmpl.name());
    act->setData(i);
    connect(act, SIGNAL(triggered()), this, SLOT(templateChanged()));
    if (!i) (QAction *) templateMenu->insertSeparator(act);
  }
}

void TexwizApp::templateChanged()
{
  if (QAction *act = qobject_cast<QAction *>(sender()))
     setTemplate(act->data().toInt());
}


/* EOF */

