/***************************************************************************
                          texwiz.h  -  description
                             -------------------
    begin                : Sam Nov 29 16:47:18 CET 2003
    copyright            : (C) 2003 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEXWIZ_H
#define TEXWIZ_H

#include "ui_texwizmain.h"
// #include "template.h"

// #ifdef USE_KDE
//   #include <kapplication.h>
//   #include <dcopclient.h>
// #endif

// include files for QT
#include <QMainWindow>
#include <QVector>

#include "template.h"

class QEvent;
class QFont;
class QLabel;
class QMenuBar;
class QMessageBox;
class QMenu;
class QSettings;
class QString;
class QStringList;
class QToolBar;
class MarginPrev;

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class TexwizView is
  * created which creates your view.
  */
class TexwizApp : public QMainWindow, private Ui::TexwizMain
{
  Q_OBJECT

  public:
    TexwizApp();
    ~TexwizApp();


//     void setKile(bool b = false);
//     void setOutputFile(const QString &of) {outputFile = of;};

  private:
    bool koma, pdf, twoside, idx, biblio;
    bool listSort, restoreSize, restorePos;
    bool useKile;
    int prev_w, prev_h, prev_x, prev_y;
    QString title, author, date, version;
    QString t_head, t_subject, t_publisher, t_thanks,
            t_dedication, t_extra, tb_top, tb_bottom;
    QString collector, workingDir, texDir, ltexDir;
    QString outputFile;
    QFont prevFont;
    QStringList letterPrivateDetails,
                letterCompanyDetails,
                customMarkups,
                headingOptions,
                oldTemplates;
    QVector<Template> *templateList;
    QMenu *templateMenu;
//     QMenuBar *MenuBar;
    QToolBar *toolBar;
    QLabel *labelMarginTop,
           *labelMarginBottom,
           *labelMarginInner,
           *labelMarginOuter;

    MarginPrev *marginPrev;

    QString collectString();
    QString collectLetter();
    QString commentHeader();
    QString collectTitle();
    QString documentClass();
    QString collectLang();
    QString usePackages();
    QString pageStyle();
    QString pageOptions();
    QString pdfOptions();
    QString customMacros();
    QString titlePage();
    void initConnections();
    void initListWidget();
    void initGui();
    void initSettings();
    void saveSettings(bool all = false);
    void closeEvent(QCloseEvent *);
    void resizeEvent(QResizeEvent *);
    void showEvent(QShowEvent *);
    void updateTemplates();

//   #ifdef USE_KDE
//     DCOPClient *client;
//   #endif

  private slots:
    void accept();
    void reject();

    void filePreview();
    void fileSaveAs();
    void helpAbout();
    void headingsOptions();
    void slotShowScrhOptions(int);
    void marginChanged(int);
    void pkgIns();
    void pkgAdd();
    void pkgRem();
    void pkgClear();
    void pkgSort();
    void preferences();
    void saveAllSettings();
    void selectBibDb();
    void editBibStyles();
//     void toggleStatusbar(bool);
//     void toggleToolbar(bool);
    void toggleKoma(bool);
    void whatsThis();
    void extendedTitles();
    void editCustomMarkup();
    void saveTemplate();
    void manageTemplates();
    void setTemplate(int);
    void templateChanged();

signals:
    void sigPageStyle(int);
    void sigResize();

};
#endif 

